# Copyright 2023 Google LLC.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import argparse

import yaml
from vizier.service import vizier_client

from trainV2 import main_train
from util.util import get_conf_list
import multiprocessing
MAX_TRIALS_PER_CLIENT = 25
MAX_PROCS = 4

"""Example of a Vizier Client, which can be run on multiple machines.

For distributed cases, this is meant to be used after the Vizier Server (see
run_vizier_server.py`) has been launched and provided an address to connect to.
Example of a launch command:

```
python run_vizier_client.py --address="localhost:[PORT]"
```

where `address` was provided by the server.

If not provided, the Vizier Server will be created locally, which still allows
parallelization via multithreading, but will not be able to coordinate jobs
across different machines.
"""

from typing import Sequence

from absl import app
from absl import flags
from absl import logging

from vizier.service import clients
from vizier.service import pyvizier as vz

def get_scale_type(sType):
    if sType=='LOG':
        return vz.ScaleType.LOG
    if sType == 'LINEAR':
        return vz.ScaleType.LINEAR
    if sType == 'REVERSE_LOG':
        return vz.ScaleType.REVERSE_LOG
    if sType == 'UNIFORM_DISCRETE':
        return vz.ScaleType.UNIFORM_DISCRETE
    return vz.ScaleType.LINEAR
def creat_client(config,server='localhost:45437',owner='a',study_id='gmnet', client_id='0'):
    clients.environment_variables.server_endpoint = server
    study_config = vz.StudyConfig()  # Search space, metrics, and algorithm.
    root = study_config.search_space.root
    if config == None:
        study = clients.Study.from_owner_and_id(owner,study_id)
        return study

    for key in config.keys():
        sType = get_scale_type(config[key]['scale_type']),
        if config[key]['type'] == 'float':
            root.add_float_param(
                key,
                min_value=config[key]['min'],
                max_value=config[key]['max'],
                scale_type=sType[0],
            )
        elif config[key]['type'] == 'int':
            root.add_int_param(
                key,
                min_value=config[key]['min'],
                max_value=config[key]['max'],
                scale_type=sType[0],
            )

    study_config.metric_information.append(
        vz.MetricInformation(
            name='mIoU',
            goal=vz.ObjectiveMetricGoal.MAXIMIZE,
            min_value=0.0,
            max_value=1.0,
        )
    )

    # study_config.algorithm = vz.Algorithm.GAUSSIAN_PROCESS_BANDIT
    study_config.algorithm = vz.Algorithm.QUASI_RANDOM_SEARCH
    # study_config.algorithm = vz.Algorithm.RANDOM_SEARCH

    client = vizier_client.create_or_load_study(
        server,
        owner_id=owner,
        client_id=client_id,
        study_id=study_id,
        study_config=study_config,
    )
    return client

def main_process(config, search_keys, client):
    print(client)
    for _ in range(MAX_TRIALS_PER_CLIENT):
        trials = client.get_suggestions(suggestion_count=1)
        for trial in trials:
            materialized_trial = trial
            print(materialized_trial)
            for key in search_keys:
                print(key)
                value = materialized_trial.parameters.get_value(key)
                print(value)
                config[key] = value
            config['trial_id'] = trial.id
            ioU = 0.3
            # ioU = main_train(config)
            m = vz.Measurement()
            m.metrics = {'mIoU': ioU}
            client.complete_trial(trial.id,m)
            logging.info(
            'Trial %d completed with metrics: %s', trial.id, m.metrics
            )

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Train MFNet with pytorch')
    parser.add_argument('--config',  '-c',  type=str, default='./config/tokyo.config')
    parser.add_argument('--gpu',  '-g',  type=int, default=-1)
    args = parser.parse_args()

    if args.config:
        file_address = open(args.config)
        config = yaml.safe_load(file_address)
    else:
        print('no config in cmd failed.!')
    search_sp = config['search_space']
    base_cfg = config['base_cfg']
    if base_cfg:
        file_address = open(base_cfg)
        base_cfg = yaml.safe_load(file_address)
    else:
        print('no base config.!')
    if args.gpu>=0:
        base_cfg['gpu_ids']=[args.gpu]
    for key in base_cfg.keys():
        if isinstance(base_cfg[key],str):
            if ',' in base_cfg[key]:
                base_cfg[key] = get_conf_list(base_cfg[key])
    # client = creat_client(search_sp, server='120.26.68.165:15707',owner='g',client_id='4')
    # client = creat_client(search_sp, server='120.26.68.165:15707',owner='h',client_id=str(args.gpu))
    client = creat_client(search_sp, server='localhost:45147',owner='h',client_id=str(args.gpu))
    main_process(base_cfg, search_sp.keys(),client)
    trial_filter = vz.TrialFilter(status=[vz.TrialStatus.COMPLETED])
    all_trials = client.list_trials()
    trials = []
    for t in filter(trial_filter, all_trials):
        trials.append(t)
    if len(trials) >= MAX_TRIALS_PER_CLIENT * MAX_PROCS:
        if client.get_study_state() == vz.StudyState.COMPLETED:
            client.set_study_state(vz.StudyState.COMPLETED)
        optimal_trials = client.list_optimal_trials()
        print(optimal_trials)
        for optimal_trial in optimal_trials:
            print(f"Pareto-optimal trial found so far has parameters {optimal_trial.parameters} and metrics {optimal_trial.final_measurement.metrics}")

